package org.openxma.dsl.pom.formatting;

import org.openxma.dsl.core.model.Import;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.core.model.ModelElement;
import org.openxma.dsl.core.model.Style;
import org.openxma.dsl.core.model.TrueLiteral;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.FeatureReference;
import org.openxma.dsl.dom.model.PresentableFeature;
import org.openxma.dsl.pom.PomSingletons;
import org.openxma.dsl.pom.model.AbsoluteSiblingAttachment;
import org.openxma.dsl.pom.model.AlignmentProperty;
import org.openxma.dsl.pom.model.AttachmentOwnPosition;
import org.openxma.dsl.pom.model.AttachmentProperty;
import org.openxma.dsl.pom.model.AttachmentSiblingPosition;
import org.openxma.dsl.pom.model.AttachmentSpecification;
import org.openxma.dsl.pom.model.Button;
import org.openxma.dsl.pom.model.Command;
import org.openxma.dsl.pom.model.ComplexElement;
import org.openxma.dsl.pom.model.Component;
import org.openxma.dsl.pom.model.ComposedElement;
import org.openxma.dsl.pom.model.Content;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.DataObjectVariable;
import org.openxma.dsl.pom.model.EventFunction;
import org.openxma.dsl.pom.model.EventMapping;
import org.openxma.dsl.pom.model.EventMappingList;
import org.openxma.dsl.pom.model.EventType;
import org.openxma.dsl.pom.model.FieldFeature;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.GuiElementEventMapping;
import org.openxma.dsl.pom.model.HorizontalAlignment;
import org.openxma.dsl.pom.model.IElementOnWhichCanBeAttached;
import org.openxma.dsl.pom.model.InitEventMapping;
import org.openxma.dsl.pom.model.Label;
import org.openxma.dsl.pom.model.LayoutDataProperty;
import org.openxma.dsl.pom.model.Page;
import org.openxma.dsl.pom.model.ParentAttachment;
import org.openxma.dsl.pom.model.ReferencedXMAComposite;
import org.openxma.dsl.pom.model.ReferencedXMAPage;
import org.openxma.dsl.pom.model.SetOfGuiElements;
import org.openxma.dsl.pom.model.Table;
import org.openxma.dsl.pom.model.TableColumn;
import org.openxma.dsl.pom.model.Text;
import org.openxma.dsl.pom.model.VerticalAlignment;
import org.openxma.dsl.pom.model.XmadslComposite;
import org.openxma.dsl.pom.model.XmadslPage;
import org.openxma.dsl.pom.naming.PomNameFunction;
import org.openxma.dsl.pom.util.LayoutData;

/**
 * Simple PomModel serializer
 *
 */
public class PomModelSerializer {

	StringBuilder buf;
	
	public static PomNameFunction getPomNameFunction() {
		return PomSingletons.getPomNameFunction();
	}		

	/**
	 * Execute the serialization
	 *
	 * @param model
	 * @return
	 */
	public String serialize(Model model) {
		buf = new StringBuilder();		
		buf.append("namespace " + model.getName()+" ");
		for (Import e : model.getImports()) {
			if (e instanceof Import) serialize((Import) e);
		}
		for (ModelElement e : model.getElements()) {
			if (e instanceof Component)
				serialize((Component) e);
		}
		return buf.toString();
	}

	protected void serialize(Import imp) {
		buf.append(" import ");
		buf.append(imp.getImportedNamespace());		
		//buf.append(imp.getImportURI());
		buf.append(" ");
	}

	protected void serialize(Component comp) {
		buf.append(" component xma::");
		buf.append(comp.getXmaComponent().getNamClass());
		buf.append(" { ");
		for (Page p : comp.getPages()) {
			if (p instanceof ReferencedXMAPage) {
				serialize((ReferencedXMAPage) p);
			} else if (p instanceof XmadslPage) {
				serialize((XmadslPage) p);
			}
		}
		buf.append(" } ");
	}

	protected void serialize(ReferencedXMAPage page) {
		buf.append(" page xma::");
		buf.append(page.getXmaPage().getNamClass());
		buf.append(" { ");
		for (DataObjectVariable d : page.getDataobjects()) {
			serialize(d);
		}
		for (Command c : page.getCommands()) {
			serialize(c);
		}
		serialize(page.getEvents());
		if (page.getContent() != null) {
			for (ComplexElement e : page.getContent().getContainerItems()) {
				if (e instanceof ReferencedXMAComposite) {
					serialize((ReferencedXMAComposite) e);
				}
			}
			/*
			for (ComposedElement e : page.getContent().getElements()) {
				if (e instanceof ReferencedXMAComposite) {
					serialize((ReferencedXMAComposite) e);
				}
			}
			 */
		}
		buf.append(" } ");
	}

	protected void serialize(XmadslPage page) {
		buf.append(" page ");
		buf.append(page.getName());
		buf.append(" { ");
		for (DataObjectVariable d : page.getDataobjects()) {
			serialize(d);
		}
		for (Command c : page.getCommands()) {
			serialize(c);
		}
		serialize(page.getEvents());
		if (page.getContent() != null) {
			for (ComposedElement e : page.getContent().getElements()) {
				if (e instanceof ReferencedXMAComposite) {
					serialize((ReferencedXMAComposite) e);
				}
			}
			/*
			for (ComposedElement e : page.getContent().getElements()) {
				if (e instanceof ReferencedXMAComposite) {
					serialize((ReferencedXMAComposite) e);
				}
			}
			 */
		}
		buf.append(" } ");
	}

	protected void serialize(DataObjectVariable d) {
		buf.append(" data ");
		buf.append(d.getType().getName());
		if (d.isIsCollection()) {
			buf.append("[]");
		}
		buf.append(" ");
		buf.append(d.getName());
	}

	protected void serialize(Command c) {
		if (c.isClientOnly()) {
			buf.append(" uicommand ");
		} else {
			buf.append(" command ");
		}
		buf.append(c.getName());
		buf.append(" ");
	}

	protected void serialize(EventMappingList el) {
		if (el != null && el.getMapping().size()>0) {
			buf.append(" eventmapping { ");
			for (EventMapping e : el.getMapping()) {
				serialize(e);
			}
			buf.append(" } ");
		}
	}

	protected void serialize(EventMapping em) {
		if (em instanceof InitEventMapping) {
			buf.append(" onInit ");
		} else if (em instanceof GuiElementEventMapping) {
			GuiElementEventMapping gem = (GuiElementEventMapping)em;
			if (gem.getControl() instanceof GuiElement) {
				GuiElement g = (GuiElement)gem.getControl();
				buf.append(getNameOfGuiElement(g));
			}
			switch (gem.getEvent().getValue()) {
			case EventType.ON_SELECTION_VALUE: buf.append(".onSelection "); break;
			case EventType.ON_DEFAULT_SELECTION_VALUE: buf.append(".onDefaultSelection "); break;
			case EventType.ON_EXPAND_VALUE: buf.append(".onExpand "); break;
			case EventType.ON_COLLAPSE_VALUE: buf.append(".onCollapse "); break;
			}
		}
		buf.append(" -> ");
		
		boolean first=true;
		for (EventFunction eventFunction : em.getEventFunctions()) {
			if (first) {
				first=false;
			} else {
				buf.append(",");
			}
			buf.append(eventFunction.getName());
		}		
		
		buf.append(" ");
	}

	protected void serialize(ReferencedXMAComposite comp) {
		if (comp.isReferencingPageComposite()) {
			buf.append(" content ");
		} else {
			buf.append(" panel xma::");
			buf.append(comp.getXmaComposite().getNamInstance());
		}
		buf.append(" { ");
		if (comp.getContent() != null) {
			serialize(comp.getContent());
		}
		buf.append(" } ");
	}

	protected void serialize(XmadslComposite comp) {
		if (comp.isHorizontalPartitioning()) {
			buf.append(" hpanel ");
		} else if (comp.isVerticalPartitioning()) {
			buf.append(" vpanel ");
		} else {
			buf.append(" panel ");
		}
		buf.append(comp.getName());
		if (comp.getLabelText() != null) {
			buf.append(" \""+comp.getLabelText().getString()+"\" ");
		}
		if (comp.getLayoutProperties() != null) {
			serialize(new LayoutData(comp));
		}
		if (comp.getScrollable() != null && comp.getScrollable() instanceof TrueLiteral) {
			buf.append(" scrollable = true ");
		}
		buf.append(" { ");
		if (comp.getContent() != null) {
			serialize(comp.getContent());
		}
		buf.append(" } ");
	}

	protected void serialize(Table table) {
		buf.append(" table ");
		buf.append(table.getName());
		buf.append(" key = ");
		buf.append(table.getObject().getName());
		buf.append(".");
		buf.append(table.getKey().getName());
		buf.append(" ");
		if (table.getLayoutProperties() != null) {
			serialize(new LayoutData(table));
		}
		buf.append(" { ");
		for (TableColumn col : table.getColumns()) {
			buf.append(" ");
			buf.append(col.getObject().getName());
			buf.append(".");
			buf.append(col.getAttribute().getName());
			if (col.getWidth()>0) {
				buf.append(" width = ");
				buf.append(col.getWidth());
			}
		}
		buf.append(" } ");
	}

	protected void serialize(Content cont) {
		for (ComposedElement e : cont.getElements()) {
			if (e instanceof ReferencedXMAComposite) {
				serialize((ReferencedXMAComposite) e);
			} else if (e instanceof XmadslComposite) {
				serialize((XmadslComposite) e);
			} else if (e instanceof Table) {
				serialize((Table)e);
			}
		}
		for (ComplexElement e : cont.getContainerItems()) {
			if (e instanceof ReferencedXMAComposite) {
				serialize((ReferencedXMAComposite) e);
			} else if (e instanceof XmadslComposite) {
				serialize((XmadslComposite) e);
			} else if (e instanceof Table) {
				serialize((Table)e);
			}
		}
		for (SetOfGuiElements s : cont.getWidgetSetItems()) {
			boolean first = true;
			for (ComposedElement e : s.getElements()) {
				if (first == false) {
					buf.append(" & ");
				}
				if (e instanceof Button) {
					serialize((Button)e);
				}
				if (e instanceof CustomizeableField) {
					serialize((CustomizeableField)e);
				}
				if (e instanceof Label) {
					serialize((Label)e);
				}
				if (e instanceof XmadslComposite) {
					serialize((XmadslComposite)e);
				}
				first=false;
			}
		}
	}

	protected void serialize(CustomizeableField cf) {
		buf.append(" ");
		buf.append(cf.getObject().getName());
		buf.append(getFeatureDefinitionString(cf.getFeature()));
		buf.append(" ");
	}

	public static String getFeatureDefinitionString(FieldFeature fieldFeature) {
		if (fieldFeature == null) {
			return null;
		}
		StringBuilder ds = new StringBuilder();
		ds.append(".");
		addFeatureSegmentString(ds,fieldFeature.getFeature());
		if (fieldFeature.getSubFeature() != null) {
			ds.append(getFeatureDefinitionString(fieldFeature.getSubFeature()));
		}
		return ds.toString();
	}

	public static void addFeatureSegmentString(StringBuilder ds, PresentableFeature feature) {
				
		if (feature.eIsProxy()) {
			String name = getPomNameFunction().apply(feature);			
			ds.append(name);
		} else if (feature instanceof Attribute) {
			Attribute h = (Attribute)feature;
			ds.append(h.getName());
		} else if (feature instanceof FeatureReference ) {
			FeatureReference fr = (FeatureReference)feature;
			ds.append(fr.getSource().getName()+"."+fr.getAttribute().getName());
			if (fr.getName() != null) {
				ds.append(" as "+fr.getName());
			}
		} else {
			throw new RuntimeException("Serializion for feature not defined: "+feature);
		}
	}

	protected void serialize(LayoutData layout) {
		for (AttachmentProperty att : layout.getAttachments()) {
			switch(att.getOwnPosition().getValue()) {
			case AttachmentOwnPosition.LEFT_VALUE: buf.append(" left = "); break;
			case AttachmentOwnPosition.RIGHT_VALUE: buf.append(" right = "); break;
			case AttachmentOwnPosition.TOP_VALUE: buf.append(" top = "); break;
			case AttachmentOwnPosition.BOTTOM_VALUE: buf.append(" bottom = "); break;
			}
			serialize(att.getAttachmentSpecification());
		}
		if (layout.getStyles().size()>0) {
			buf.append(" style= ");
			boolean first=true;
			for (Style style : layout.getStyles()) {
				if (!first) buf.append(",");
				buf.append(style.getName());
				first=false;
			}
			buf.append(" ");
		}
		for (LayoutDataProperty ldp : layout.getProperties()) {
			if (ldp instanceof AlignmentProperty) {
				AlignmentProperty alp = (AlignmentProperty)ldp;
				buf.append(" align= ");
				if (alp.getHAlignment() != null) {
					switch (alp.getHAlignment().getValue()) {
					case HorizontalAlignment.RIGHT_VALUE: buf.append("right "); break;
					case HorizontalAlignment.LEFT_VALUE: buf.append("left "); break;
					}
				} else if (alp.getVAlignment() != null) {
					switch (alp.getVAlignment().getValue()) {
					case VerticalAlignment.TOP_VALUE: buf.append("top "); break;
					case VerticalAlignment.BOTTOM_VALUE: buf.append("bottom "); break;
					}
				}
			}

		}
	}

	protected void serialize(AttachmentSpecification attSpec) {
		if (attSpec instanceof ParentAttachment) {
			ParentAttachment pa = (ParentAttachment)attSpec;
			if (pa.isRelative()) {
				buf.append(pa.getDistance());
				buf.append("%");
			} else {
				buf.append(pa.getDistance()); // TODO: check this line
			}
		} else if (attSpec instanceof AbsoluteSiblingAttachment) {
			AbsoluteSiblingAttachment sa = (AbsoluteSiblingAttachment)attSpec;
			serialize(sa.getSibling());
			switch(sa.getSiblingPosition().getValue()) {
			case AttachmentSiblingPosition.TOP_VALUE: buf.append(".top"); break;
			case AttachmentSiblingPosition.BOTTOM_VALUE: buf.append(".bottom"); break;
			case AttachmentSiblingPosition.LEFT_VALUE: buf.append(".left"); break;
			case AttachmentSiblingPosition.RIGHT_VALUE: buf.append(".right"); break;
			}
		}
	}

	protected void serialize(IElementOnWhichCanBeAttached elem) {
		if (elem instanceof GuiElement) {
			GuiElement ge = (GuiElement)elem;
			String name = getNameOfGuiElement(ge);
			if (name != null) {
				buf.append(name);
			}
		}
	}

	protected String getNameOfGuiElement(GuiElement ge) {
		if (ge instanceof XmadslComposite) {
			return ((XmadslComposite)ge).getName();
		} else if (ge instanceof Button) {
			return ((Button)ge).getName();
		} else if (ge instanceof Text) {
			return ((Text)ge).getName();
		}  else if (ge instanceof Table) {
			return ((Table)ge).getName();
		}

		throw new RuntimeException("The method 'getName' is not supported for the GuiElement: "+ge);
	}

	protected void serialize(Button but) {
		buf.append(" button ");
		buf.append(but.getName());
		buf.append(" \"");
		buf.append(but.getLabelText().getString());
		buf.append("\" ");
	}

	protected void serialize(Label label) {
		buf.append(" label ");
		buf.append(label.getName());
		buf.append(" \"");
		buf.append(label.getLabelText().getString());
		buf.append("\" ");
	}
}
