package org.openxma.dsl.pom.scoping;

import org.openxma.dsl.pom.PomPackage;
import org.openxma.dsl.pom.model.EventType;
import org.openxma.dsl.pom.model.IGuiElementWithEvent;



public class EnumerationFilter {
	
	public enum EnumSupport {
		SUPPORTED,NOT_SUPPORTED,DEPRECATED,NOT_PART_OF_ENUMERATION;
	}
		
	/**
	 * Supported EventTypes in GuiElementEventMapping.
	 * Either eventType or eventTypeName has to be defined. In case that 
	 * eventType is not defined it is derived from the eventTypeName.
	 * 
	 * @param control
	 * @param eventType
	 * @param eventTypeName
	 * @return
	 */
	public static EnumSupport supportGuiElementEventMapping_EventType(IGuiElementWithEvent control, EventType eventType, String eventTypeName) {
		if (control == null) return EnumSupport.NOT_SUPPORTED;
		if (eventType == null) {
			if (eventTypeName == null) {
				throw new IllegalArgumentException("Either eventType or eventTypeName has to be defined.");
			}
			eventType = EventType.getByName(eventTypeName);
			if (eventType == null) return EnumSupport.NOT_PART_OF_ENUMERATION;
		}
		
		int controlClassId = control.eClass().getClassifierID();
		switch(controlClassId) {
		case PomPackage.BUTTON:
			switch(eventType.getValue()) {
			case EventType.ON_CLICK_VALUE: return EnumSupport.SUPPORTED;	
			case EventType.ON_SELECTION_VALUE: return EnumSupport.DEPRECATED;						
			}
			break;
		case PomPackage.TABLE:
			switch(eventType.getValue()) {
			case EventType.ON_CLICK_VALUE: return EnumSupport.SUPPORTED;	
			case EventType.ON_SELECTION_VALUE: return EnumSupport.SUPPORTED;	
			case EventType.ON_DOUBLE_CLICK_VALUE: return EnumSupport.SUPPORTED;	
			case EventType.ON_DEFAULT_SELECTION_VALUE: return EnumSupport.DEPRECATED;			
			}
			break;
		case PomPackage.CUSTOMIZEABLE_FIELD:
			switch(eventType.getValue()) {
			case EventType.ON_CLICK_VALUE: return EnumSupport.SUPPORTED;	
			case EventType.ON_SELECTION_VALUE: return EnumSupport.DEPRECATED;						
			}
			break;
		case PomPackage.TREE:
			switch(eventType.getValue()) {
			case EventType.ON_CLICK_VALUE: return EnumSupport.SUPPORTED;	
			case EventType.ON_SELECTION_VALUE: return EnumSupport.SUPPORTED;
			case EventType.ON_DOUBLE_CLICK_VALUE: return EnumSupport.SUPPORTED;	
			case EventType.ON_DEFAULT_SELECTION_VALUE: return EnumSupport.DEPRECATED;
			case EventType.ON_EXPAND_VALUE: return EnumSupport.SUPPORTED;	
			case EventType.ON_COLLAPSE_VALUE: return EnumSupport.SUPPORTED;					
			}
			break;
		case PomPackage.COMBO:
			switch(eventType.getValue()) {
			case EventType.ON_CLICK_VALUE: return EnumSupport.SUPPORTED;	
			case EventType.ON_SELECTION_VALUE: return EnumSupport.SUPPORTED;					
			}
			break;
		case PomPackage.LABEL:
			switch(eventType.getValue()) {
			case EventType.ON_CLICK_VALUE: return EnumSupport.SUPPORTED;					
			}
			break;
		case PomPackage.IMAGE:
			switch(eventType.getValue()) {
			case EventType.ON_CLICK_VALUE: return EnumSupport.SUPPORTED;
			case EventType.ON_DOUBLE_CLICK_VALUE: return EnumSupport.SUPPORTED;			
			case EventType.ON_SELECTION_VALUE: return EnumSupport.DEPRECATED;			
			case EventType.ON_DEFAULT_SELECTION_VALUE: return EnumSupport.DEPRECATED;			
			}
			break;			
		}
		return EnumSupport.NOT_SUPPORTED;
	}	
}
